@@ -12,7 +12,7 @@ use XSLoader;
 # Use David Golden's version numbering suggestions. Note that we have to call
 # the XSLoader before evaling the version string because XS modules check the
 # version *string*, not the version *number*, at boot time.
-our $VERSION = "0.40_01";
+our $VERSION = "0.40_02";
 XSLoader::load('C::Blocks', $VERSION);
 $VERSION = eval $VERSION;
 
@@ -30,7 +30,6 @@ sub import {
 	*{$caller.'::csub'} = sub () {};
 	*{$caller.'::cshare'} = sub () {};
 	*{$caller.'::clex'} = sub () {};
-	*{$caller.'::cisa'} = sub () {};
 	_import();
 }
 
@@ -66,70 +65,6 @@ END {
 	_cleanup();
 }
 
-########################################################################
-                   package C::Blocks::Type::NV;
-########################################################################
-use Scalar::Util;
-use Carp;
-
-our $TYPE = 'NV';
-our $INIT = 'SvNV';
-our $CLEANUP = 'sv_setnv';
-
-sub check_var_types {
-	my $package = shift @_;
-	$@ = '';
-	while (@_) {
-		my ($arg_name, $arg) = splice @_, 0, 2;
-		$@ .= "$arg_name is not defined\n" and next if not defined $arg;
-		$@ .= "$arg_name is a reference\n" and next if ref($arg);
-		$@ .= "$arg_name does not look like a number" and next
-			unless Scalar::Util::looks_like_number($arg);
-	}
-	if ($@ eq '') {
-		undef $@;
-		return 1;
-	}
-	return 0;
-}
-
-########################################################################
-               package C::Blocks::Type::double;
-########################################################################
-our $TYPE = 'double';
-our $INIT = 'SvNV';
-our $CLEANUP = 'sv_setnv';
-*check_var_types = \&C::Blocks::Type::NV::check_var_types;
-
-########################################################################
-               package C::Blocks::Type::float;
-########################################################################
-our $TYPE = 'float';
-our $INIT = 'SvNV';
-our $CLEANUP = 'sv_setnv';
-*check_var_types = \&C::Blocks::Type::NV::check_var_types;
-
-########################################################################
-               package C::Blocks::Type::int;
-########################################################################
-our $TYPE = 'int';
-our $INIT = 'SvIV';
-our $CLEANUP = 'sv_setiv';
-*check_var_types = \&C::Blocks::Type::NV::check_var_types;
-
-########################################################################
-               package C::Blocks::Type::uint;
-########################################################################
-our $TYPE = 'unsigned int';
-our $INIT = 'SvUV';
-our $CLEANUP = 'sv_setuv';
-# Should check sign, too
-*check_var_types = \&C::Blocks::Type::NV::check_var_types;
-
-# Other types:
-# int2ptr
-# uint2ptr
-
 1;
 
 __END__
@@ -202,10 +137,13 @@ C::Blocks - embeding a fast C compiler directly into your Perl parser
  use C::Blocks;
  use C::Blocks::PerlAPI;
  cshare {
-     /* This function can be imported into other lexical scopes. */
+     /* These functions can be imported into other lexical scopes. */
      void say_hi() {
          printf("Hello from My::Fastlib\n");
      }
+     void My::Fastlib::goodbye() {
+         printf("Goodbye from My::Fastlib\n");
+     }
  }
  
  1;
@@ -217,6 +155,7 @@ C::Blocks - embeding a fast C compiler directly into your Perl parser
  
  cblock {
      say_hi();
+     My::Fastlib::goodbye();
  }
  
  ### Use Perl to generate code at compile time
@@ -307,22 +246,26 @@ This produces the output
  After the cblock, message is [5.938]
  and array contains 7
 
-An important low-level detail is that the actual variable name for the 
-SV*, AV*, or HV* in your C code is based on the original variable name 
-with some gentle mangling. This lets you use C-side variables with the 
-"same" name (sans the sigil):
+It often happens that the scalars you want to work with represent a
+specific piece of information with a specific data type. In that case,
+you do not want to manipulate the SV but instead want to work directly
+with the data represented within the SV. If you indicate the type during
+the variable's declaration, C::Blocks can use that type information to
+automatically unpack and repack the data for you:
 
- my $N = 100;
- my $result;
+ # Need to pull in the type packages
+ use C::Blocks::Types qw(double Int);
+ my double $sum = 0;
+ my Int $limit = 100;
  cblock {
-     int i;
-     int result = 0;
-     int N = SvIV($N); /* notice "same" name N */
-     for (i = 1; i < N; i++) result += i;
-     sv_setiv($result, result);
+     for (int i = 1; i < $limit; i++) {
+         $sum += 1.0 / i;
+	 }
  }
- print "The brute-force sum from 1 to 100 is $result\n";
- print "Gauss would have said ", $N * ($N - 1) / 2, "\n";
+ print "The sum of 1/x for x from 1 to $limit is $sum\n";
+
+The details for how types work, and how to create your own, are
+discussed under L<C::Blocks/TYPES>.
 
 =head2 Private C Declarations
 
@@ -526,6 +469,13 @@ lexically scoped variables, however. Changes to package-scoped variables
 (including lexically scoped names, i.e. C<our $package_var>) persist,
 as would be expected if these variables were set in BEGIN blocks.
 
+=head2 Filtering C Code
+
+In addition to generating raw C code, you can modify code before it is
+compiled with a filter module. Filter modules are given the complete
+contents of the code in the underbar variable C<$_> before it is
+processed through the compiler. See L<C::Blocks::Filter> for more details.
+
 =head2 Configuring the Compiler
 
 Sometimes you need to configure the compiler. The most common situation
@@ -577,6 +527,30 @@ The warnings are handled using Perl's built-in warnings system, so as
 with all warnings, the reporting of compiler and linker warnings can be
 controlled lexically.
 
+=head2 Double-colons
+
+Double colons are a standard notation for package-name resolution in
+Perl. They are also invalid syntax in C. For this reason, C::Blocks lets
+you use double-colons in your C code by performing a simple replacement.
+Whenever C::Blocks encounters a double-colon that is not part of a
+sigiled variable name or the contents of a string, it will replace the
+pair with a pair of underscores. Thus this code:
+
+ void My::Func() { ... }
+
+actually gets compiled as
+
+ void My__Func() { ... }
+
+Note that this replacement does not modify the contents of strings, so
+if you spit out a printed message such as
+
+  printf("There was a problem in My::Module::method...\n");
+
+the double-colons will pass through unscathed. In addition, this
+replacement occurs with text provided by interpolation blocks and code
+provided for type initialization and cleanup.
+
 =head1 PERFORMANCE
 
 C<C::Blocks> is not a silver bullet for performance. Other Perl 
@@ -678,10 +652,6 @@ precede a block of C code encapsulated in curly brackets. Because these use the
 Perl keyword API, they parse the C code during Perl's parse stage, so any code
 errors in your C code will be caught during parse time, not during run time.
 
-In addition to these keywords, C<C::Blocks> lets you indicate types and
-type conversion with C<cisa>. Unlike the other keywords, this keyword is
-not followed by a block of code, but the type and a list of variables.
-
 =over
 
 =item cblock { code }
@@ -696,14 +666,20 @@ declarations are not allowed. Also, variable declarations and preprocessor
 definitions are confined to the C<cblock> and will not be present in later
 C<cblock>s. For that sort of behavior, see C<clex>.
 
-Variables with C<$> sigils are interpreted as referring to the C<SV*>
-representing the variable in the current lexical scope, unless otherwise
-specified with a C<cisa> statement.
+By default, variables with C<$> sigils are interpreted as referring to
+the C<SV*> representing the variable in the current lexical scope. The
+exception is when a variable is declared with a type, a la
+
+ my Class::Name $thing;
+
+Here C<Class::Name> specifies the type of C<$thing>. The package
+C<Class::Name> has information used by C::Blocks to unpack and repack
+C<$thing> into the appropriate C data type. Implementation details are
+discussed under L<C::Blocks/TYPES>.
 
 Note: If you need to leave a C<cblock> early, you should use a C<return>
-statement without any arguments. This will also bypass the data repacking
-provided by C<cisa> types.
-
+statement without any arguments. In particular, this will bypass any
+cleanup code provided by types.
 
 =item clex { code }
 
@@ -727,41 +703,186 @@ C code contained in a csub block is wrapped into an xsub function definition.
 This means that after this code is compiled, it is accessible just like any
 other xsub.
 
-Currently, C<csub> does not work.
+=back
 
-=item cisa type variable-list
+=head1 TYPES
 
-If you include sigil variables in your C<cblock> blocks (not C<clex>,
-C<cshare>, or C<csub>, just C<cblock>), they will normally be resolved
-to the underlying SV data structure for that variable. Under many
-circumstances, you do not need to manipulate the SV itself, but merely
-need the data contained in the SV (or the object pointed to by the SV).
-A C<cisa> statement tells C::Blocks that certain variables should be
-represented by a C data structure other than an SV. The package used for
-the type (must) have package constants that indicate the C type to use,
-and how to marshall the data at the beginning and end of your block.
+Perl lets you declare the type of a lexically scoped variable with
+syntax like this:
 
-C<cisa> statements also have the runtime responsibility of validating
-the data in the variables. Failed validations should probably throw
-exceptions indicating which variables did not satisfy validation, and
-why they failed. Your validation code can make as much or as little
-noise as you deem appropriate, from quietly setting C<$@> to warning to
-throwing exceptions. Note that you could include validation code in the
-initialization function, but C<cisa> validation is only called once per
-C<cisa> statement, whereas the variable initialization code is called
-at the beginning of each C<cblock> that uses the variable.
+ my Some::Class $foo;
 
-Packages that represent types must include the package variables
-C<$TYPE> and C<$INIT>. The first indicates the C type while the second
-indicates a C macro or function that accepts an SV and returns the
-data of type C<$TYPE>. C<$CLEANUP> is an optional macro or function that
-takes the original SV and the (presumably revised) data, and updates the
-contents of the SV. Runtime type checking is performed by the package
-method C<check_var_types>, which gets key/value pairs of the variable
-name and the variable.
+Here, Perl knows that C<$foo> is of type C<Some::Class>. However, it
+does not let C<Some::Class> do much with C<$foo>, rendering this feature
+mostly useless. It is useful for C::Blocks, however, because C::Blocks
+I<can> use the type of a scalar: it consults that package for
+information about how to unpack and repack C<$foo> into a more useful
+C-side data type.
+
+Type classes for basic C data types (like C<double> and C<long>) are
+provided in L<C::Blocks::Types>. In this section, I discuss how you
+might create your own type class that can be utilized by C::Blocks.
+
+When a type-annotated sigiled variable is used in a C<cblock>, 
+C::Blocks looks for a function C<c_blocks_init_cleanup> in the type 
+package and calls it. This class method is expected to produce 
+initialization code for the variable, and optionally to produce cleanup 
+code as well. This code is added to the beginning and end of the larger 
+chunk of C code that eventually gets compiled.
+
+To understand how all of this works, let's examine what C::Blocks does 
+when we do not specify a type for our scalar. Here is a way to 
+manipulate the floating-point value of a Perl variable using C::Blocks:
+
+ my $foo;
+ cblock {
+     sv_setnv($foo, 3.14159);
+ }
+
+The actual C code that gets produced for the C<cblock> is a void
+function that looks essentially like this:
+
+ void op_func() {
+     SV * _PERL_SCALAR_foo = (SV*)PAD_SV(3);
+     sv_setnv(_PERL_SCALAR_foo, 3.14159);
+ }
+
+In the first line of this function, the SV associated with C<$foo> is 
+retrieved from the current PAD. The next line is exactly what we typed 
+in the C<cblock>, except that C<$foo> has been replaced with 
+C<_PERL_SCALAR_foo>.
+
+When we use types, we can say what we mean more directly:
+
+ use C::Blocks::Types;
+ my C::Blocks::Type::double $foo = 0;
+ cblock {
+     $foo = 3.14159;
+ }
+
+Indented for clarity, the actual code that gets produced for the 
+C<cblock> this looks like:
+
+ void op_func() {
+     SV * SV__PERL_SCALAR_foo = (SV*)PAD_SV(3);
+     double _PERL_SCALAR_foo
+         = SvNV(SV__PERL_SCALAR_foo);
+     
+     _PERL_SCALAR_foo = 3.14159;
+     
+     sv_setnv(SV__PERL_SCALAR_foo,
+         _PERL_SCALAR_foo);
+ }
+
+The midddle line of code, C<_PERL_SCALAR_foo = 3.14159>, clearly
+comes from the contents of the C<cblock>. The rest was supplied by the
+C<c_blocks_init_cleanup> method of the C<C::Blocks::Type::double>
+package. This example illustrates what this method is supposed to do.
+
+The C<c_blocks_init_cleanup> method is supposed to return a string with
+initialization code and, optionally, another string containing cleanup
+code. In order to construct that code, the method is called with the
+following arguments:
+
+=over
+
+=item package name
+
+The type's package name. In this case it would be
+C<C::Blocks::Type::double>.
+
+=item C variable name
+
+The long-winded semi-mangled variable name. In this case, for the 
+variable C<$foo>, we got the name C<_PERL_SCALAR_foo>. This 
+string of characters is injected into the cblock wherever it encounters 
+C<$foo>, so the code must declare a variable with this name.
+
+=item sigil type
+
+The C struct type for this associated sigil. If the sigil is C<$>, we'll
+get C<SV>, for C<@>, we'll get C<AV>, and for C<%>, we'll get C<HV>.
+
+=item pad offset
+
+The integer offset of the variable in the current pad.
 
 =back
 
+The method must use this information to produce a string containing 
+initialization code. In this example, the generated code is a sequence 
+of C declarations that culminate in C<_PERL_SCALAR_foo> being a 
+variable of type C<double> and being assigned the current 
+floating-point value of the Perl variable C<$foo>. It also produces a 
+string containing "cleanup" code, code that modifies C<$foo> with the 
+value of C<_PERL_SCALAR_foo> as the block comes to a close.
+
+If your type only needs to unpack a value, and does not need to perform
+any cleanup, then a rudimentary template for C<c_blocks_init_cleanup>
+could look like this:
+
+ sub c_blocks_init_cleanup {
+     my ($package, $C_name, $sigil_type, $pad_offset) = @_;
+     
+     # Should probably die if $sigil_type is not SV
+     # but I can't handle that quite yet. :-(
+     
+     # Assumes that some_type and get_some_type_from_SV
+     # have been defined in some clex or cshare block:
+     return qq{
+         some_type * $C_name
+             = get_some_type_from_SV(PAD_SV($pad_offset));
+     };
+ }
+
+The most critical pieces are that C<PAD_SV($pad_offset)> must be used to
+retrieve the SV, and we must interpolate the C<$C_name> as the name of
+the C-side variable.
+
+If some part of the initialization process allocates resources that need
+to be cleaned up when the block comes to a close, that code should be
+returned as the second return value of the method:
+
+ sub c_blocks_init_cleanup {
+     my ($package, $C_name, $sigil_type, $pad_offset) = @_;
+     
+     # Should probably die if $sigil_type is not SV
+     # but I can't handle that quite yet. :-(
+     
+     # Assumes that some_type and get_some_type_from_SV
+     # have been defined in some clex or cshare block:
+     my $init_code = qq{
+         FILE *fp_$C_name = fopen("logfile_$C_name", "a");
+         some_type * $C_name
+             = get_some_type_from_SV(PAD_SV($pad_offset));
+     };
+     my $cleanup_code = qq{
+	     fclose(fp_$C_name);
+	 }
+	 return ($init_code, $cleanup_code);
+ }
+
+In this case, the user would not only have access to their variable as
+the provided C type, but they would have an additional file handle that
+they could use for logging. A C<cblock> using this would be able to say
+
+ my Some::Type $foo;
+ cblock {
+     fprintf(fp_$foo, "about to call big-operation\n");
+     
+     big_operation($foo, 5, "hello");
+     
+     fprintf(fp_$foo, "Finished big-operation\n");
+ }
+
+Of course, this example is a bit contrived: you should probably handle
+this sort of logging at the Perl level. However, it gives you some idea
+for how you could use cleanup code together with initialization code.
+
+(Eventually include a full example of a class that wraps itself around
+a C struct, including all necessary cshare statements, so that the
+init code's call to special functions make sense.)
+
 =head1 SEE ALSO
 
 This module uses a special fork of the Tiny C Compiler. The fork is 
